@@ -1,22 +1,62 @@
-# TODO: token 相关的初始化可以通过 default Callable 来实现
-import logging
+"""
+This module provides functionalities for managing datasets,
+including uploading files to cloud storage.
+
+The module is designed to interact with cloud services such as AWS S3 using `boto3` and
+also supports HTTP requests via `requests`.
+It defines several dataclasses and utilities to facilitate the handling, uploading, and
+managing of dataset files within a specified environment.
+
+Key Classes:
+- DatasetFileForUpload: A dataclass representing a file to be uploaded,
+including its local and remote paths.
+- Dataset: A class to handle dataset-related operations.
+- DatasetCommit: Manages commits related to datasets.
+- DatasetFile: Represents individual files within a dataset.
+- DatasetList: Manages a list of datasets.
+- UploadInfo: Contains metadata related to the upload process.
+
+Environment Variables:
+- `HEYWHALE_HOST`: The base URL of the Heywhale platform (default: "https://www.heywhale.com").
+- `HEYWHALE_DS_BUCKET`: The name of the bucket used for datasets (default: "kesci").
+
+Logging:
+- A logger is initialized for tracking the module's activities and errors.
+
+Dependencies:
+- boto3: For interacting with AWS services.
+- requests: For making HTTP requests.
+- requests_unixsocket: For making HTTP requests over UNIX domain sockets.
+"""
+
 import os
+from pathlib import Path
 import random
 import string
-import time
 from dataclasses import dataclass, field
-from datetime import datetime
-from pathlib import Path
+from urllib.parse import quote
 from typing import BinaryIO, Dict, List, Optional, Set, Union
+import boto3  # type: ignore
+from boto3.s3.transfer import TransferConfig
+import requests  # type: ignore
+import requests_unixsocket  # type: ignore
+from botocore.exceptions import (  # type: ignore
+    ClientError,
+    NoCredentialsError,
+    PartialCredentialsError,
+)
+from dotenv import load_dotenv
 
-import boto3
-import requests
-from botocore.exceptions import ClientError, NoCredentialsError, PartialCredentialsError
 
-from .dataset import Dataset, DatasetCommit, DatasetFile, DatasetList, UploadInfo
+from mw_python_sdk.core import Dataset, DatasetCommit, DatasetFile, UploadInfo
+from mw_python_sdk.utils import (
+    logger,
+    parse_datetime,
+    convert_to_dataset_file,
+    generate_timestamped_string,
+)
 
-# Create a logger for your library
-logger = logging.getLogger(__name__)
+load_dotenv()  # take environment variables from local .env
 
 HEYWHALE_SITE = os.getenv("HEYWHALE_HOST", "https://www.heywhale.com")
 HEYWHALE_DS_BUCKET = os.getenv("HEYWHALE_DS_BUCKET", "kesci")
@@ -104,48 +144,9 @@ class DatasetConstructor:
         return dataset
 
 
-def parse_datetime(date_string: str) -> datetime:
-    """
-    Parse a datetime string into a datetime object.
-
-    Args:
-        date_string (str): The datetime string to parse.
-
-    Returns:
-        datetime: A datetime object.
-
-    """
-    if date_string is None:
-        return datetime.now()
-    return datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%S.%fZ")
-
-
-def convert_to_dataset_file(files) -> List[DatasetFile]:
-    """
-    将给定的文件列表转换为 DatasetFile 对象的列表。
-
-    Args:
-        files (List[Dict]): 文件列表，每个元素为字典类型，包含 "_id"、"Token"、"Size" 和 "SubPath" 等字段。
-
-    Returns:
-        List[DatasetFile]: 转换后的 DatasetFile 对象列表。
-
-    """
-    if files is None:
-        return []
-    dataset_files = [
-        DatasetFile(
-            file.get("_id"),
-            file.get("Token"),
-            file.get("Size"),
-            "" if file.get("SubPath") is None else file.get("SubPath"),
-        )
-        for file in files
-    ]
-    return dataset_files
-
-
-def get_dataset(dataset_id: str, token: Optional[str] = None) -> Optional[Dataset]:
+def get_dataset(
+    dataset_id: str, commit: Optional[str] = None, token: Optional[str] = None
+) -> Optional[Dataset]:
     """
     Fetches dataset details from Heywhale.
 
@@ -159,41 +160,27 @@ def get_dataset(dataset_id: str, token: Optional[str] = None) -> Optional[Datase
     Raises:
         ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
-
+    token = _init_token(token)
     url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}"
     headers = {
         "x-kesci-token": token,
         "x-kesci-resource": dataset_id,
     }
+    params = {}
+    if commit is not None:
+        params["Version"] = commit
 
-    response = requests.get(url, headers=headers, timeout=10)
+    response = requests.get(url, headers=headers, params=params, timeout=10)
     if response.status_code == 200:
-        # print("get datasets ", response.json())
+        logger.debug("get datasets %s", response.json())
         files = convert_to_dataset_file(response.json().get("Files"))
-        # print("Get Files ", files)
         files_with_subpath = convert_to_dataset_file(
             response.json().get("FilesStructure")
         )
-        # print("Get Files with subpath", files_with_subpath)
-        # filter the files overlapped with files_with_subpath
         files_key_set = set([file.key for file in files_with_subpath])
         for file in files:
-            # print("file", file)
             if file.key not in files_key_set:
-                # print("not in")
                 files_with_subpath.append(file)
-        # print("files_with_subpath", files_with_subpath)
-        # files.extend(files_with_subpath)
-        # print("return dataset files",files)
-
-        # Files里面不是某个版本的Files
-        # 这里用 downloadUrl?Version的方式重新构造一次 files
         return Dataset(
             _id=response.json().get("_id"),
             title=response.json().get("Title"),
@@ -216,6 +203,28 @@ def get_dataset(dataset_id: str, token: Optional[str] = None) -> Optional[Datase
     return None
 
 
+def _init_token(token: Optional[str]):
+    """
+    获取初始化令牌
+
+    Args:
+        token (Optional[str]): 令牌，如果为None，则从环境变量'MW_TOKEN'中获取
+
+    Returns:
+        str: 初始化令牌
+
+    Raises:
+        ValueError: 如果未提供令牌且环境变量'MW_TOKEN'未设置时，引发该异常
+    """
+    if token is None:
+        token = os.getenv("MW_TOKEN")
+        if not token:
+            raise ValueError(
+                "No token provided and 'MW_TOKEN' environment variable is not set."
+            )
+    return token
+
+
 def _update_dataset(
     dataset_id: str,
     files: List[DatasetFile],
@@ -230,12 +239,7 @@ def _update_dataset(
     :param token: The token for authentication. If not provided, the function will use the 'MW_TOKEN' environment variable.
     :raises ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}/files"
     headers = {"x-kesci-token": token, "x-kesci-resource": dataset_id}
     data = {
@@ -272,12 +276,7 @@ def _get_update_token(token: Optional[str] = None) -> Optional[UploadInfo]:
     Raises:
         ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     url = f"{HEYWHALE_SITE}/api/dataset-upload-token"
     headers = {"x-kesci-token": token}
     response = requests.get(url, headers=headers, timeout=10)
@@ -296,21 +295,9 @@ def _get_update_token(token: Optional[str] = None) -> Optional[UploadInfo]:
         return None
 
 
-def generate_timestamped_string(revision: int) -> str:
-    """
-    Generates a timestamped string based on the current time and a revision number.
-
-    :param revision: The revision number.
-    :return: A timestamped string.
-    """
-    timestamp = int(time.time() * 1000)
-    result = f"{timestamp}_{revision}"
-    return result
-
-
 def _upload_file(
     path_or_fileobj: Union[str, Path, bytes, BinaryIO],
-    path_in_dataset: str | Path,
+    path_in_dataset: Union[str, Path],
     upload_info: UploadInfo,
 ) -> str:
     """
@@ -330,6 +317,7 @@ def _upload_file(
         aws_session_token=upload_info.token,
         region_name=upload_info.region,
     )
+
     s3 = session.client("s3", endpoint_url=upload_info.endpoint)
 
     bucket_name = upload_info.bucket
@@ -343,21 +331,24 @@ def _upload_file(
         # 目录存在api的SubPath字段里面
         # 这里仿照前端的格式存以防有什么不统一的地方导致错误
         object_key = os.path.join(object_key, os.path.basename(path_in_dataset))
-    # print(f"upload object key {object_key}")
     try:
         if isinstance(path_or_fileobj, (str, Path)):
-            with open(path_or_fileobj, "rb") as file:
-                s3.put_object(Bucket=bucket_name, Key=object_key, Body=file)
+            # s3 大小限制，使用multipart上传。
+            # https://docs.aws.amazon.com/AmazonS3/latest/userguide/qfacts.html
+            multipart_config = TransferConfig(multipart_chunksize=1024 * 1024)
+            s3.upload_file(
+                path_or_fileobj,bucket_name, object_key,Config=multipart_config
+            )
         else:
             s3.put_object(Bucket=bucket_name, Key=object_key, Body=path_or_fileobj)
-    except Exception as e:
+    except (IOError, s3.exceptions.S3UploadFailedError) as e:
         print(f"Error putting object '{object_key}' from bucket '{bucket_name}': {e}")
     return object_key
 
 
 def upload_file(
     path_or_fileobj: Union[str, Path, bytes, BinaryIO],
-    path_in_dataset: str | Path,
+    path_in_dataset: Union[str, Path],
     dataset_id: str,
     overwrite: bool = False,
     token: Optional[str] = None,
@@ -375,12 +366,7 @@ def upload_file(
     Raises:
         ValueError: If no token is provided and the 'MW_TOKEN' environment variable is not set.
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
 
     upload_info = _get_update_token(token)
     if upload_info is None:
@@ -428,7 +414,7 @@ def _norm_path(pp: str) -> str:
 
 
 def upload_folder(
-    folder_path: str | Path,
+    folder_path: Union[str, Path],
     folder_in_dataset: str,
     dataset_id: str,
     overwrite: bool = False,
@@ -453,12 +439,7 @@ def upload_folder(
                     or if the dataset is not found,
                     or if failed to get upload token.
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     # check folder existence
     if not os.path.isdir(folder_path):
         raise ValueError("Folder does not exist.")
@@ -471,36 +452,26 @@ def upload_folder(
         raise ValueError(f"Dataset '{dataset_id}' not found.")
 
     for root, _dirs, files in os.walk(folder_path):
-        # print("files")
         for file in files:
-            # print(f"file {file}")
             file_path = os.path.join(root, file)
             rel_path = os.path.relpath(os.path.join(root, file), folder_path)
             path_in_dataset = os.path.join(folder_in_dataset, rel_path)
-            # print(f"root {root} rel_path {rel_path} upload file {file_path} path in {os.path.join(path_in_dataset,rel_path)} id {id}")
             object_key = _upload_file(file_path, path_in_dataset, upload_info)
             skip_upload = False
             for dfile in dataset.files:
-                # print(
-                #     f"dfile {dfile.path_in_dataset() } file {path_in_dataset}"
-                # )
                 if dfile.path_in_dataset() != path_in_dataset:
-                    # print(f"add dfile {dfile}")
                     new_dataset_files.add(dfile)
                 elif not overwrite:
-                    # print("file exists, skip uploading")
                     skip_upload = True
                     break
             if skip_upload:
                 break
             else:
                 pp = str(Path(path_in_dataset).parent)
-                # print(f"Append new file {object_key}, {_norm_path(pp)}")
                 new_dataset_files.add(DatasetFile("", object_key, 0, _norm_path(pp)))
     if len(new_dataset_files) == 0:
         # skip update if no files in directory
         return
-    # print(new_dataset_files)
     _update_dataset(
         dataset_id,
         list(new_dataset_files),
@@ -509,7 +480,7 @@ def upload_folder(
     )
 
 
-def _init_cache(cache_dir: Optional[str | Path] = None) -> Path:
+def _init_cache(cache_dir: Optional[Union[str, Path]] = None) -> Path:
     """
     Initializes the cache directory for the MW SDK.
 
@@ -529,6 +500,75 @@ def _init_cache(cache_dir: Optional[str | Path] = None) -> Path:
     return cache_dir
 
 
+def _download_single_file_by_cached(
+    dataset_id: str,
+    endpoint: str,
+    ak: str,
+    sk: str,
+    token: str,
+    bucket_and_key: str,
+    commit: str,
+    filename: str,
+    cache_dir: Path,
+    local_dir: Optional[Union[str, Path]] = None,
+    local_path: Optional[Union[str, Path]] = None,
+) -> Path:
+    """
+    Downloads a single file from a dataset.
+    Args:
+        dataset_id: The ID of the dataset.
+        endpoint: The endpoint of the object storage service.
+        ak: The access key ID.
+        sk: The secret access key.
+        token: The session token.
+        etag: The ETag of the file.
+        commit: The commit id.
+        filename: The name of the file.
+        cache_dir: The cache directory.
+        local_dir: The local directory to save the file to. If not provided, the default cache directory will be used.
+        local_path: The path to save the file to. If specified, `local_dir` is ignored.
+    Returns:
+        The path to the downloaded
+    """
+    slk_path = (
+        Path(cache_dir)
+        .expanduser()
+        .joinpath("datasets")
+        .joinpath(dataset_id)
+        .joinpath("snapshots")
+        .joinpath(commit)
+        .joinpath(filename)
+    )
+    if local_dir is not None:
+        slk_path = Path(local_dir).joinpath(filename)
+    if local_path is not None:
+        slk_path = Path(local_path)
+    if slk_path.exists():
+        return slk_path
+    mw_cached_root_dir = os.environ.get("MW_CACHED_ROOT_DIR", "/var/mw")
+    sock_path = os.path.join(mw_cached_root_dir, "cached.sock")
+    session = requests_unixsocket.Session()
+    # convert a path to percent-encoded path
+    quoted_path = quote(sock_path, safe="")
+    logger.debug("connecting to mw-cached at %s", quoted_path)
+    r = session.get(
+        f"http+unix://{quoted_path}/get/object",
+        params={
+            "uri": f"s3://{bucket_and_key}",
+            "ak": ak,
+            "sk": sk,
+            "token": token,
+            "endpoint": endpoint,
+        },
+        timeout=3600,
+    )
+    blob_path = r.json().get("cachedFilePath")
+    logger.debug("cached blob file path: %s", blob_path)
+    slk_path.parent.mkdir(parents=True, exist_ok=True)
+    slk_path.symlink_to(blob_path)
+    return slk_path
+
+
 def _download_single_file(
     download_url: str,
     id: str,
@@ -536,8 +576,8 @@ def _download_single_file(
     etag: str,
     commit: str,
     cache_dir: Path,
-    local_dir: Optional[str | Path] = None,
-    local_path: Optional[str | Path] = None,
+    local_dir: Optional[Union[str, Path]] = None,
+    local_path: Optional[Union[str, Path]] = None,
 ) -> Path:
     """
     Downloads a single file from a dataset.
@@ -573,6 +613,7 @@ def _download_single_file(
         slk_path = Path(local_path)
     if slk_path.exists():
         return slk_path
+
     blob_path = Path(cache_dir).expanduser().joinpath("blobs").joinpath(etag)
     if not blob_path.exists():
         file_path = (
@@ -586,17 +627,17 @@ def _download_single_file(
         with open(file_path, "wb") as file:
             for chunk in response.iter_content(chunk_size=chunk_size):
                 file.write(chunk)
-        blob_path = file_path.rename(file_path.parent.joinpath(etag))
+            file_path.rename(file_path.parent.joinpath(etag))
     slk_path.parent.mkdir(parents=True, exist_ok=True)
     slk_path.symlink_to(blob_path)
     return slk_path
 
 
 def download_dir(
-    id: str,
+    dataset_id: str,
     commit: Optional[str] = None,
-    cache_dir: Optional[str | Path] = None,
-    local_dir: Optional[str | Path] = None,
+    cache_dir: Optional[Union[str, Path]] = None,
+    local_dir: Optional[Union[str, Path]] = None,
     token: Optional[str] = None,
 ) -> Path:
     """Download a directory from the dataset.
@@ -607,33 +648,20 @@ def download_dir(
     Returns:
         str: The path to the downloaded directory.
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     cache_dir = _init_cache(cache_dir)
-    dataset_detail = get_dataset(id)
+    dataset_detail = get_dataset(dataset_id)
     if dataset_detail is None:
-        raise ValueError(f"Dataset '{id}' not found.")
+        raise ValueError(f"Dataset '{dataset_id}' not found.")
     path_to_key: Dict[str, str] = dict()
-    # print("files", dataset_detail.files)
     for file in dataset_detail.files:
-        # print("pathind ", file.path_in_dataset())
         path_to_key[file.path_in_dataset()] = file.key
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
-    session = boto3.Session(
-        aws_access_key_id=upload_info.ak,
-        aws_secret_access_key=upload_info.sk,
-        aws_session_token=upload_info.token,
-        region_name=upload_info.region,
-    )
-    s3 = session.client("s3", endpoint_url=upload_info.endpoint)
+
     bucket_name = upload_info.bucket
-    url = f"{HEYWHALE_SITE}/api/datasets/{id}/downloadUrl"
+    url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}/downloadUrl"
     headers = {
         "x-kesci-token": token,
     }
@@ -643,73 +671,92 @@ def download_dir(
     response = requests.get(url, headers=headers, params=params, timeout=10)
     if response.status_code == 200:
         files = response.json().get("files")
-        for file in files:
-            download_url = file.get("Url")
-            filepath_in_dataset = file.get("Name") + file.get("Ext")
-            sub_path = file.get("SubPath")
+        for file_json in files:
+            download_url = file_json.get("Url")
+            filepath_in_dataset = file_json.get("Name") + file_json.get("Ext")
+            sub_path = file_json.get("SubPath")
             if sub_path is not None:
                 filepath_in_dataset = os.path.join(sub_path, filepath_in_dataset)
             key = path_to_key[filepath_in_dataset]
-            # get etag
-            try:
-                # print("Get etag bucket key", bucket_name, key)
-                response = s3.head_object(Bucket=bucket_name, Key=key)
-                # Extract the ETag from the response
-                etag = response["ETag"]
-                # print(f"Etag {etag}")
-                # trim quote
-                etag = etag[1:-1]
-                _download_single_file(
-                    download_url,
-                    id,
-                    filepath_in_dataset,
-                    etag,
+            mw_cached_root_dir = os.environ.get("MW_CACHED_ROOT_DIR", "/var/mw")
+            sock_path = os.path.join(mw_cached_root_dir, "cached.sock")
+            # file exists
+            if os.path.exists(sock_path):
+                _download_single_file_by_cached(
+                    dataset_id,
+                    upload_info.endpoint,
+                    upload_info.ak,
+                    upload_info.sk,
+                    upload_info.token,
+                    os.path.join(bucket_name, key),
                     commit,
+                    filepath_in_dataset,
                     cache_dir,
                     local_dir,
                 )
-            except NoCredentialsError:
-                print("Error: AWS credentials not available.")
-            except PartialCredentialsError:
-                print("Error: Incomplete AWS credentials.")
-            except ClientError as e:
-                # The client error includes more details about the error
-                if e.response["Error"]["Code"] == "404":
-                    print(
-                        f"Error: The object '{key}' does not exist in the bucket '{bucket_name}'."
+            else:
+                # get etag
+                try:
+                    session = boto3.Session(
+                        aws_access_key_id=upload_info.ak,
+                        aws_secret_access_key=upload_info.sk,
+                        aws_session_token=upload_info.token,
+                        region_name=upload_info.region,
                     )
-                elif e.response["Error"]["Code"] == "403":
-                    print(
-                        f"Error: Access to the object '{key}' in the bucket '{bucket_name}' is forbidden."
+                    s3 = session.client("s3", endpoint_url=upload_info.endpoint)
+                    response = s3.head_object(Bucket=bucket_name, Key=key)
+                    # Extract the ETag from the response
+                    etag = response["ETag"]
+                    etag = etag[1:-1]
+                    _download_single_file(
+                        download_url,
+                        dataset_id,
+                        filepath_in_dataset,
+                        etag,
+                        commit,
+                        cache_dir,
+                        local_dir,
                     )
-                else:
-                    print(f"Error: {e.response['Error']['Message']}")
-            except Exception as e:
-                print(f"An unexpected error occurred: {str(e)}")
+                except NoCredentialsError:
+                    print("Error: AWS credentials not available.")
+                except PartialCredentialsError:
+                    print("Error: Incomplete AWS credentials.")
+                except ClientError as e:
+                    # The client error includes more details about the error
+                    if e.response["Error"]["Code"] == "404":
+                        print(
+                            f"Error: The object '{key}' does not exist in the bucket '{bucket_name}'."
+                        )
+                    elif e.response["Error"]["Code"] == "403":
+                        print(
+                            f"Error: Access to the object '{key}' in the bucket '{bucket_name}' is forbidden."
+                        )
+                    else:
+                        print(f"Error: {e.response['Error']['Message']}")
+                except Exception as e:
+                    print(f"An unexpected error occurred: {str(e)}")
     else:
         print(response.text)
         response.raise_for_status()
     if local_dir is None:
-        dir = (
+        return (
             Path(cache_dir)
             .expanduser()
             .joinpath("datasets")
-            .joinpath(id)
+            .joinpath(dataset_id)
             .joinpath("snapshots")
             .joinpath(commit)
         )
-        return dir
     else:
-        dir = Path(local_dir)
-        return dir
+        return Path(local_dir)
 
 
 def download_file(
     dataset_id: str,
     filename: str,
     commit: Optional[str] = None,
-    cache_dir: Optional[str | Path] = None,
-    local_dir: Optional[str | Path] = None,
+    cache_dir: Optional[Union[str, Path]] = None,
+    local_dir: Optional[Union[str, Path]] = None,
     token: Optional[str] = None,
 ) -> Path:
     """Download a file from the dataset.
@@ -723,12 +770,7 @@ def download_file(
     Returns:
         str: The path to the downloaded file.
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     cache_dir = _init_cache(cache_dir)
     dataset_detail = get_dataset(dataset_id)
     if dataset_detail is None:
@@ -742,13 +784,7 @@ def download_file(
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
-    session = boto3.Session(
-        aws_access_key_id=upload_info.ak,
-        aws_secret_access_key=upload_info.sk,
-        aws_session_token=upload_info.token,
-        region_name=upload_info.region,
-    )
-    s3 = session.client("s3", endpoint_url=upload_info.endpoint)
+
     bucket_name = upload_info.bucket
 
     url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}/downloadUrl"
@@ -763,45 +799,76 @@ def download_file(
     if response.status_code == 200:
         files = response.json().get("files")
         logger.debug("get dataset files through api: %s", files)
-        for file in files:
-            sub_path = file.get("SubPath") if file.get("SubPath") is not None else ""
-            if os.path.join(sub_path, file.get("Name")) + file.get("Ext") == filename:
-                download_url = file.get("Url")
+        for file_json in files:
+            sub_path = (
+                file_json.get("SubPath") if file_json.get("SubPath") is not None else ""
+            )
+            if (
+                os.path.join(sub_path, file_json.get("Name")) + file_json.get("Ext")
+                == filename
+            ):
+                download_url = file_json.get("Url")
                 break
         else:
-            raise ValueError(f"File '{filename}' not found in dataset '{id}'.")
+            raise ValueError(f"File '{filename}' not found in dataset '{dataset_id}'.")
     else:
         print(response.text)
         response.raise_for_status()
     key = path_to_key[filename]
-    # get etag
-    try:
-        response = s3.head_object(Bucket=bucket_name, Key=key)
-        etag = response["ETag"]
-        etag = etag[1:-1]
-        logger.debug("get object %s etag: %s", os.path.join(bucket_name, key), etag)
-        slk_path = _download_single_file(
-            download_url, dataset_id, filename, etag, commit, cache_dir, local_dir
+    mw_cached_root_dir = os.environ.get("MW_CACHED_ROOT_DIR", "/var/mw")
+    sock_path = os.path.join(mw_cached_root_dir, "cached.sock")
+    # file exists
+    if os.path.exists(sock_path):
+        logger.debug("cached socket exists using cached")
+        return _download_single_file_by_cached(
+            dataset_id,
+            upload_info.endpoint,
+            upload_info.ak,
+            upload_info.sk,
+            upload_info.token,
+            os.path.join(bucket_name, key),
+            commit,
+            filename,
+            cache_dir,
+            local_dir,
         )
-    except NoCredentialsError:
-        print("Error: AWS credentials not available.")
-    except PartialCredentialsError:
-        print("Error: Incomplete AWS credentials.")
-    except ClientError as e:
-        # The client error includes more details about the error
-        if e.response["Error"]["Code"] == "404":
-            print(
-                f"Error: The object '{key}' does not exist in the bucket '{bucket_name}'."
+    else:
+        # get etag
+        try:
+            session = boto3.Session(
+                aws_access_key_id=upload_info.ak,
+                aws_secret_access_key=upload_info.sk,
+                aws_session_token=upload_info.token,
+                region_name=upload_info.region,
             )
-        elif e.response["Error"]["Code"] == "403":
-            print(
-                f"Error: Access to the object '{key}' in the bucket '{bucket_name}' is forbidden."
+            s3 = session.client("s3", endpoint_url=upload_info.endpoint)
+            response = s3.head_object(Bucket=bucket_name, Key=key)
+            etag = response["ETag"]
+            etag = etag[1:-1]
+            logger.debug("get object %s etag: %s", os.path.join(bucket_name, key), etag)
+            slk_path = _download_single_file(
+                download_url, dataset_id, filename, etag, commit, cache_dir, local_dir
             )
-        else:
-            print(f"Error: {e.response['Error']['Message']}")
-    except Exception as e:
-        print(f"An unexpected error occurred: {str(e)}")
-    return slk_path
+            return slk_path
+        except NoCredentialsError:
+            print("Error: AWS credentials not available.")
+        except PartialCredentialsError:
+            print("Error: Incomplete AWS credentials.")
+        except ClientError as e:
+            # The client error includes more details about the error
+            if e.response["Error"]["Code"] == "404":
+                print(
+                    f"Error: The object '{key}' does not exist in the bucket '{bucket_name}'."
+                )
+            elif e.response["Error"]["Code"] == "403":
+                print(
+                    f"Error: Access to the object '{key}' in the bucket '{bucket_name}' is forbidden."
+                )
+            else:
+                print(f"Error: {e.response['Error']['Message']}")
+        except Exception as e:
+            print(f"An unexpected error occurred: {str(e)}")
+        return Path("")
 
 
 def delete_file(
@@ -810,15 +877,10 @@ def delete_file(
     token: Optional[str] = None,
 ):
     """Delete a file from the dataset."""
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     dataset = get_dataset(dataset_id, token)
     if dataset is None:
-        raise ValueError(f"Dataset '{id}' not found.")
+        raise ValueError(f"Dataset '{dataset_id}' not found.")
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
@@ -852,12 +914,7 @@ def create_commit(
     commit_message: str,
     token: Optional[str] = None,
 ) -> DatasetCommit:
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     upload_info = _get_update_token(token)
     if upload_info is None:
         raise ValueError("Failed to get upload token.")
@@ -874,8 +931,8 @@ def create_commit(
 def create_dataset(
     title: str,
     local_dir: str,
-    remote_dir: Union[str | None],
-    short_description: Union[str | None],
+    remote_dir: Optional[str],
+    short_description: Optional[str],
 ) -> Optional[Dataset]:
     """
     根据提供的参数创建数据集对象
@@ -923,12 +980,7 @@ def _create_dataset(
     Returns:
         Dataset: the created dataset
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
     url = f"{HEYWHALE_SITE}/api/datasets"
     headers = {
         "x-kesci-token": token,
@@ -1001,12 +1053,7 @@ def delete_dataset(
         HTTPError: 如果请求失败，则抛出HTTPError异常。
 
     """
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
+    token = _init_token(token)
 
     url = f"{HEYWHALE_SITE}/api/datasets/{dataset_id}"
     headers = {
@@ -1023,168 +1070,3 @@ def delete_dataset(
     print(response.text)
     response.raise_for_status()
     return
-
-
-def _list_datasets(
-    title: str = "",
-    limit: int = 10,
-    page: int = 1,
-    start_date: time = None,
-    end_date: time = None,
-    token: Optional[str] = None,
-):
-    if token is None:
-        token = os.getenv("MW_TOKEN")
-        if not token:
-            raise ValueError(
-                "No token provided and 'MW_TOKEN' environment variable is not set."
-            )
-    headers = {
-        "x-kesci-token": token,
-    }
-
-    url = f"{HEYWHALE_SITE}/api/user/datasets"
-    params = {
-        "perPage": limit,
-        "page": page,
-        "Title": title,
-        "startDate": start_date,
-        "endDate": end_date,
-    }
-
-    response = requests.get(
-        url,
-        headers=headers,
-        params=params,
-        timeout=10,
-    )
-    if response.status_code == 200:
-        document = response.json()
-        datasets: List[Dataset] = []
-        for d in document.get("data"):
-            files = convert_to_dataset_file(d.get("Files"))
-            files_with_subpath = convert_to_dataset_file(d.get("FilesStructure"))
-            if len(files) > 0:
-                files_key_set = set([file.key for file in files])
-                for file in files:
-                    if file.key not in files_key_set:
-                        files_with_subpath.append(file)
-            commits = []
-            if d.get("DatasetVersions") is not None:
-                commits = [
-                    DatasetCommit(
-                        commit.get("_id"),
-                        commit.get("CommitMessage"),
-                        parse_datetime(commit.get("CreateDate")),
-                    )
-                    for commit in d.get("DatasetVersions")
-                ]
-            datasets.append(
-                Dataset(
-                    _id=d.get("_id"),
-                    title=d.get("Title"),
-                    short_description=d.get("ShortDescription"),
-                    folder_name=d.get("FolderName"),
-                    files=files_with_subpath,
-                    commits=commits,
-                    created_at=parse_datetime(d.get("CreateDate")),
-                    updated_at=parse_datetime(d.get("UpdateDate")),
-                )
-            )
-        return DatasetList(
-            datasets=datasets,
-            total=document.get("totalNum"),
-            page=document.get("page"),
-            limit=document.get("perPage"),
-        )
-    else:
-        print(response.text)
-        response.raise_for_status()
-        return None
-
-
-class DatasetIterator:
-    """
-    DatasetIterator 类
-
-    该类提供了一个迭代器接口，用于遍历数据集。
-    它封装了数据集加载、预处理和迭代的逻。
-    """
-
-    def __init__(self, title, page=1, limit=10):
-        self.title = title
-        self.limit = limit
-        self.page = page
-        self.total = 0
-        self.processed_count = 0
-        self.current_batch = []
-        self.finished = False
-
-    def __iter__(self):
-        return self
-
-    def __next__(self):
-        if not self.current_batch and not self.finished:
-            # Fetch the next page of datasets
-            dataset_list = _list_datasets(
-                title=self.title,
-                page=self.page,
-                limit=self.limit,
-            )
-            self.total = dataset_list.total
-
-            if not dataset_list.datasets:
-                self.finished = True
-                raise StopIteration
-
-            self.current_batch = dataset_list.datasets
-            self.processed_count += len(self.current_batch)
-            logger.debug("Processed: %d / Total: %d", self.processed_count, self.total)
-            # If all items have been processed, mark as finished
-            if self.processed_count >= self.total:
-                self.finished = True
-            else:
-                self.page += 1
-
-        if not self.current_batch:
-            raise StopIteration
-
-        # Return the next dataset in the current batch
-        return self.current_batch.pop(0)
-
-
-def list_all_datasets(title: str = "") -> List[Dataset]:
-    """
-    列出所有满足给定标题的数据集
-
-    Args:
-        title (str, optional): 数据集标题，默认为空字符串。
-
-    Returns:
-        List[Dataset]: 包含所有满足给定标题的数据集的列表。
-
-    """
-    datasets = list()
-    for dataset in DatasetIterator(title):
-        datasets.append(dataset)
-    return datasets
-
-
-def list_datasets(
-    title: str = "",
-    limit: int = 10,
-    page: int = 1,
-) -> DatasetIterator:
-    """
-    返回数据集的迭代器对象，可指定数据集标题、返回数据条数及页数
-
-    Args:
-        title (str, optional): 数据集标题. Defaults to "".
-        limit (int, optional): 每页返回的数据条数. Defaults to 10.
-        page (int, optional): 页码. Defaults to 1.
-
-    Returns:
-        DatasetIterator: 数据集的迭代器对象.
-
-    """
-    return DatasetIterator(title, page, limit)
